--LOAD--
function love.load()

	setShorcuts()
	setMouse()
	setBackground()
	setObjects()
	setBallPs()
	setColors()
end

--UPDATE--
function love.update(dt)

	bouncePaddle(
	
		ball.x,
		ball.y,
		paddle.x,
		paddle.y,
		paddle.width,
		paddle.height
	)
	
	bounceWall(
	
		ball.x,
		ball.y,
		wall.left,
		wall.right,
		wall.up
	)

	moveBall(
	
		ball.x,
		ball.y,
		ball.dir_x,
		ball.dir_y,
		ball.vel_x,
		ball.vel_y,
		dt
	)
	
	movePaddle(
	
		love.mouse.getX(),
		paddle.width,
		wall.left,
		wall.right
	)
	
	lostLife()
end

--DRAW--
function love.draw()

	drawBackground()
	
	drawBall(
	
		ball.x,
		ball.y,
		ball.r
	)

	drawWall(
	
		wall.left,
		wall.right,
		wall.up)
		
	drawPaddle(
	
		paddle.x,
		paddle.y,
		paddle.width,
		paddle.height
	)

	info(
	
		ball.x,
		ball.y,
		ball.dir_x,
		ball.dir_y,
		ball.vel_x,
		ball.vel_y,
		paddle.x
	)
end

--KEYPRESSED--
function love.keypressed(key)

	if key == "escape"
	
	or key == "return" then
	
		love.event.quit()
	end
end

--SETSHORCUTS--
function setShorcuts()

	mo = love.mouse
	gr = love.graphics
	
	getW = gr.getWidth()
	getH = gr.getHeight()
end

--SETMOUSE--
function setMouse()

	mo.setVisible(false)
	mo.setGrab(true)
end

--SETBACKGROUND--
function setBackground()

	gr.setBackgroundColor(0x40, 0x60, 0x80)
	background = gr.newImage("background.jpg")
end

--SETOBJECTS--
function setObjects()

	mx = love.mouse.getX()
	my = getH/8

	paddle = {

		x = mx;
		y = getH - my;
		width = 0x80;
		height = 0x10;
	}
	
	ball = {

		x = mx;
		y = getH-my;
		dir_x = 1;
		dir_y = -1;
		vel_x = 0x100;
		vel_y = 0x200;
	}
	
	wall = {

		left = 8;
		right = 8;
		up = 8;
	}
end

--SETCOLORS--
function setColors()

	colors = {

		paddle = {
		
			fill = {
			
				0x40, 0x60, 0x80, 0xC0
			};
			
			line = {
			
				0x40, 0x80, 0xff, 0xE0
			};
		};
		
		info = {
		
			0xff, 0xff, 0xff
		};
		
		wall = {
		
			left = {
			
				0x40, 0x60, 0x80, 0x80
			};
			
			right = {
			
				0x40, 0x60, 0x80, 0x80
			};
			
			up = {
			
				0x40, 0x60, 0x80, 0x80
			};
		};
	}
	
	ball_ps:setColors(
		0x80, 0x60, 0x40, 0xff,
		0x40, 0x00 , 0x00 , 0x00
	)
end

--SETBALLPS--
function setBallPs()

	part = gr.newImage("part.png")
	
	ball_ps =
		gr.newParticleSystem(
			part, 0x20
		)
		
	ball_ps:setEmissionRate(0x80)
	ball_ps:setParticleLife(.2)
	
	ball_ps:setSizes(1, .5)
	ball_ps:setSpeed(0x0, 0x80)
	ball_ps:setSpread(15)
end

--MOVEBALL--
function moveBall(x, y, dx, dy, vx, vy, dt)

	ball.x = x + dx * vx * dt
	ball.y = y + dy * vy * dt
	
	ball_ps:setPosition(
	
		ball.x,
		ball.y
	)
	
	ball_ps:update(dt)
end

--BOUNCEWALL--
function bounceWall(x,	y, l, r, u)

	if x < l then
	
		ball.dir_x = 1
	
	elseif x > getW - r then
	
		ball.dir_x = -1
	end
	
	if y < u then
	
		ball.dir_y = 1
	end
end

--MOVEPADDLE--
function movePaddle(mx, w, l, r)
	
	local p = mx - w/2
	
	if p < l then
	
		p = l
	
	elseif p + w > getW - r then
	
		p = getW - r - w
	end
	
	paddle.x = p
end

--BOUNCEPADDLE--
function bouncePaddle(bx, by, px, py, pw, ph)

	if by > py then
	
		if by < py + ph then
		
			if bx > px
			
			and bx < px + pw then
			
				ball.dir_y = -1
			end
		end
		
		if bx > px
		
		and bx < px + pw/16
		
		and by < py + ph then
		
			ball.dir_x = -1
		end
		
		if bx < px + pw
		
		and bx > px + 15*pw/16
		
		and by < py + ph then
		
			ball.dir_x = 1
		end		
	end
end

--LOSTLIFE--
function lostLife()

	if ball.y > getH then

		newLife()
	end
end

--NEWLIFE--
function newLife()

	ball.x = paddle.x + paddle.width/2
	ball.y = paddle.y
	ball.dir_y = -1
end

--DRAWBALL--
function drawBall(x, y)

	gr.setBlendMode("additive")
	gr.draw(ball_ps, 0, 0)
end

--DRAWWALL--
function drawWall(l, r, u)

	gr.setBlendMode("alpha")

	gr.setColor(colors.wall.left)

	gr.rectangle(
		"fill",
		0, 0,
		l, getH
	)

	gr.setColor(colors.wall.right)

	gr.rectangle(
		"fill",
		getW - r, 0,
		r, getH
	)
	
	gr.setColor(colors.wall.up)

	gr.rectangle(
		"fill",
		l, 0,
		getW - ( l + r ), u
	)
end

--DRAWPADDLE--
function drawPaddle(x, y, w, h)

	gr.setColorMode("modulate")
	gr.setBlendMode("alpha")

	gr.setColor(colors.paddle.fill)

	gr.rectangle(
		"fill",
		x, y,
		w, h
	)
	
	gr.setColor(0, 0, 0, 255)

	gr.setLineWidth(4)
	
	gr.rectangle(
		"line",
		x, y,
		w, h
	)

	gr.setColor(colors.paddle.line)
	
	gr.setLineWidth(2)
	
	gr.rectangle(
		"line",
		x, y,
		w, h
	)
end

--DRAWBACKGROUND--
function drawBackground()

	gr.setColorMode("modulate")
	gr.setBlendMode("alpha")

	gr.draw(background, 0, 0)

end

--INFO--
function info(bx, by, dx, dy, vx, vy, px)

	gr.setColor(colors.info)

	gr.print(
	
	
		"FPS : "..
		love.timer.getFPS()..
		"\nparticules "..
		ball_ps:count()..
		"\nball.x "..
		math.floor(bx)..
		"\nball.y "..
		math.floor(by)..
		"\nball.dx "..
		dx..
		"\nball.dy "..
		dy..
		"\nball.vx "..
		vx..
		"\nball.vy "..		
		vy..
		"\npaddle.x "..
		math.floor(px),
		32, 32
	)	
end
